@@ -300,6 +300,8 @@ enum litest_device_type {
 	LITEST_DELL_CANVAS_TOTEM,
 	LITEST_DELL_CANVAS_TOTEM_TOUCH,
 	LITEST_WACOM_ISDV4_4200_PEN,
+	LITEST_ALPS_3FG,
+	LITEST_ELAN_TABLET,
 };
 
 #define LITEST_DEVICELESS	-2
@@ -437,40 +439,40 @@ void litest_set_log_handler_bug(struct libinput *libinput);
 void
 _litest_add(const char *name,
 	    const char *funcname,
-	    void *func,
+	    const void *func,
 	    int64_t required_feature,
 	    int64_t excluded_feature);
 void
 _litest_add_ranged(const char *name,
 		   const char *funcname,
-		   void *func,
+		   const void *func,
 		   int64_t required,
 		   int64_t excluded,
 		   const struct range *range);
 void
 _litest_add_for_device(const char *name,
 		       const char *funcname,
-		       void *func,
+		       const void *func,
 		       enum litest_device_type type);
 void
 _litest_add_ranged_for_device(const char *name,
 			      const char *funcname,
-			      void *func,
+			      const void *func,
 			      enum litest_device_type type,
 			      const struct range *range);
 void
 _litest_add_no_device(const char *name,
 		      const char *funcname,
-		      void *func);
+		      const void *func);
 void
 _litest_add_ranged_no_device(const char *name,
 			     const char *funcname,
-			     void *func,
+			     const void *func,
 			     const struct range *range);
 void
 _litest_add_deviceless(const char *name,
 		       const char *funcname,
-		       void *func);
+		       const void *func);
 
 struct litest_device *
 litest_create_device(enum litest_device_type which);
@@ -761,6 +763,10 @@ litest_is_switch_event(struct libinput_event *event,
 		       enum libinput_switch sw,
 		       enum libinput_switch_state state);
 
+struct libinput_event_tablet_tool *
+litest_is_proximity_event(struct libinput_event *event,
+			  enum libinput_tablet_tool_proximity_state state);
+
 void
 litest_assert_key_event(struct libinput *li, unsigned int key,
 			enum libinput_key_state state);
@@ -1149,4 +1155,12 @@ litest_send_file(int sock, int fd)
 	return write(sock, buf, n);
 }
 
+static inline int litest_slot_count(struct litest_device *dev)
+{
+	if (dev->which == LITEST_ALPS_3FG)
+		return 2;
+
+	return libevdev_get_num_slots(dev->evdev);
+}
+
 #endif /* LITEST_H */
