@@ -81,6 +81,8 @@ struct record_device {
 	struct list link;
 	char *devnode;		/* device node of the source device */
 	struct libevdev *evdev;
+	struct libevdev *evdev_prev; /* previous value, used for EV_ABS
+					deltas */
 	struct libinput_device *device;
 
 	struct event *events;
@@ -194,10 +196,18 @@ noiprintf(const struct record_context *ctx, const char *format, ...)
 	assert(rc != -1 && (unsigned int)rc > 0);
 }
 
-static inline void
-print_evdev_event(struct record_context *ctx, struct input_event *ev)
+static inline uint64_t
+time_offset(struct record_context *ctx, uint64_t time)
 {
-	const char *cname;
+	return ctx->offset ? time - ctx->offset : 0;
+}
+
+static inline void
+print_evdev_event(struct record_context *ctx,
+		  struct record_device *dev,
+		  struct input_event *ev)
+{
+	const char *tname, *cname;
 	bool was_modified = false;
 	char desc[1024];
 	uint64_t time = input_event_time(ev) - ctx->offset;
@@ -208,6 +218,7 @@ print_evdev_event(struct record_context *ctx, struct input_event *ev)
 	if (!ctx->show_keycodes)
 		was_modified = obfuscate_keycode(ev);
 
+	tname = libevdev_event_type_get_name(ev->type);
 	cname = libevdev_event_code_get_name(ev->type, ev->code);
 
 	if (ev->type == EV_SYN && ev->code == SYN_MT_REPORT) {
@@ -230,9 +241,87 @@ print_evdev_event(struct record_context *ctx, struct input_event *ev)
 			cname,
 			ev->value,
 			dt);
-	} else {
-		const char *tname = libevdev_event_type_get_name(ev->type);
+	} else if (ev->type == EV_ABS) {
+		int oldval = 0;
+		enum { DELTA, SLOT_DELTA, NO_DELTA } want = DELTA;
+		int delta = 0;
 
+		/* We want to print deltas for abs axes but there are a few
+		 * that we don't care about for actual deltas because
+		 * they're meaningless.
+		 *
+		 * Also, any slotted axis needs to be printed per slot
+		 */
+		switch (ev->code) {
+		case ABS_MT_SLOT:
+			libevdev_set_event_value(dev->evdev_prev,
+						 ev->type,
+						 ev->code,
+						 ev->value);
+			want = NO_DELTA;
+			break;
+		case ABS_MT_TRACKING_ID:
+		case ABS_MT_BLOB_ID:
+			want = NO_DELTA;
+			break;
+		case ABS_MT_TOUCH_MAJOR ... ABS_MT_POSITION_Y:
+		case ABS_MT_PRESSURE ... ABS_MT_TOOL_Y:
+			if (libevdev_get_num_slots(dev->evdev_prev) > 0)
+				want = SLOT_DELTA;
+			break;
+		default:
+			break;
+		}
+
+		switch (want) {
+		case DELTA:
+			oldval = libevdev_get_event_value(dev->evdev_prev,
+							  ev->type,
+							  ev->code);
+			libevdev_set_event_value(dev->evdev_prev,
+						 ev->type,
+						 ev->code,
+						 ev->value);
+			break;
+		case SLOT_DELTA: {
+			int slot = libevdev_get_current_slot(dev->evdev_prev);
+			oldval = libevdev_get_slot_value(dev->evdev_prev,
+							 slot,
+							 ev->code);
+			libevdev_set_slot_value(dev->evdev_prev,
+						slot,
+						ev->code,
+						ev->value);
+			break;
+		}
+		case NO_DELTA:
+			break;
+
+		}
+
+		delta = ev->value - oldval;
+
+		switch (want) {
+		case DELTA:
+		case SLOT_DELTA:
+			snprintf(desc,
+				 sizeof(desc),
+				 "%s / %-20s %6d (%+d)",
+				 tname,
+				 cname,
+				 ev->value,
+				 delta);
+			break;
+		case NO_DELTA:
+			snprintf(desc,
+				 sizeof(desc),
+				 "%s / %-20s %6d",
+				 tname,
+				 cname,
+				 ev->value);
+			break;
+		}
+	} else {
 		snprintf(desc,
 			 sizeof(desc),
 			 "%s / %-20s %6d%s",
@@ -273,18 +362,19 @@ handle_evdev_frame(struct record_context *ctx, struct record_device *d)
 	while (libevdev_next_event(evdev,
 				   LIBEVDEV_READ_FLAG_NORMAL,
 				   &e) == LIBEVDEV_READ_STATUS_SUCCESS) {
-		uint64_t time;
+		uint64_t time = input_event_time(&e);
 
 		if (ctx->offset == 0)
-			ctx->offset = input_event_time(&e);
+			ctx->offset = time;
+		else
+			time = time_offset(ctx, time);
 
 		if (d->nevents == d->events_sz)
 			resize(d->events, d->events_sz);
 
 		event = &d->events[d->nevents++];
 		event->type = EVDEV;
-		time = input_event_time(&e);
-		input_event_set_time(&e, time - ctx->offset);
+		event->time = time;
 		event->u.evdev = e;
 		count++;
 
@@ -373,9 +463,7 @@ buffer_key_event(struct record_context *ctx,
 		abort();
 	}
 
-	time = ctx->offset ?
-		libinput_event_keyboard_get_time_usec(k) - ctx->offset : 0;
-
+	time = time_offset(ctx, libinput_event_keyboard_get_time_usec(k));
 	state = libinput_event_keyboard_get_key_state(k);
 
 	key = libinput_event_keyboard_get_key(k);
@@ -415,9 +503,7 @@ buffer_motion_event(struct record_context *ctx,
 		abort();
 	}
 
-	time = ctx->offset ?
-		libinput_event_pointer_get_time_usec(p) - ctx->offset : 0;
-
+	time = time_offset(ctx, libinput_event_pointer_get_time_usec(p));
 	event->time = time;
 	snprintf(event->u.libinput.msg,
 		 sizeof(event->u.libinput.msg),
@@ -450,8 +536,7 @@ buffer_absmotion_event(struct record_context *ctx,
 		abort();
 	}
 
-	time = ctx->offset ?
-		libinput_event_pointer_get_time_usec(p) - ctx->offset : 0;
+	time = time_offset(ctx, libinput_event_pointer_get_time_usec(p));
 
 	event->time = time;
 	snprintf(event->u.libinput.msg,
@@ -483,8 +568,7 @@ buffer_pointer_button_event(struct record_context *ctx,
 		abort();
 	}
 
-	time = ctx->offset ?
-		libinput_event_pointer_get_time_usec(p) - ctx->offset : 0;
+	time = time_offset(ctx, libinput_event_pointer_get_time_usec(p));
 	button = libinput_event_pointer_get_button(p);
 	state = libinput_event_pointer_get_button_state(p);
 
@@ -519,8 +603,7 @@ buffer_pointer_axis_event(struct record_context *ctx,
 		abort();
 	}
 
-	time = ctx->offset ?
-		libinput_event_pointer_get_time_usec(p) - ctx->offset : 0;
+	time = time_offset(ctx, libinput_event_pointer_get_time_usec(p));
 	if (libinput_event_pointer_has_axis(p,
 				LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL)) {
 		h = libinput_event_pointer_get_axis_value(p,
@@ -590,8 +673,7 @@ buffer_touch_event(struct record_context *ctx,
 		abort();
 	}
 
-	time = ctx->offset ?
-		libinput_event_touch_get_time_usec(t) - ctx->offset : 0;
+	time = time_offset(ctx, libinput_event_touch_get_time_usec(t));
 
 	if (etype != LIBINPUT_EVENT_TOUCH_FRAME) {
 		slot = libinput_event_touch_get_slot(t);
@@ -674,8 +756,7 @@ buffer_gesture_event(struct record_context *ctx,
 		abort();
 	}
 
-	time = ctx->offset ?
-		libinput_event_gesture_get_time_usec(g) - ctx->offset : 0;
+	time = time_offset(ctx, libinput_event_gesture_get_time_usec(g));
 	event->time = time;
 
 	switch (etype) {
@@ -858,10 +939,7 @@ buffer_tablet_tool_proximity_event(struct record_context *ctx,
 	}
 
 	prox = libinput_event_tablet_tool_get_proximity_state(t);
-
-	time = ctx->offset ?
-		libinput_event_tablet_tool_get_time_usec(t) - ctx->offset : 0;
-
+	time = time_offset(ctx, libinput_event_tablet_tool_get_time_usec(t));
 	axes = buffer_tablet_axes(t);
 
 	idx = 0;
@@ -917,9 +995,7 @@ buffer_tablet_tool_button_event(struct record_context *ctx,
 
 	button = libinput_event_tablet_tool_get_button(t);
 	state = libinput_event_tablet_tool_get_button_state(t);
-
-	time = ctx->offset ?
-		libinput_event_tablet_tool_get_time_usec(t) - ctx->offset : 0;
+	time = time_offset(ctx, libinput_event_tablet_tool_get_time_usec(t));
 
 	event->time = time;
 	snprintf(event->u.libinput.msg,
@@ -972,10 +1048,7 @@ buffer_tablet_tool_event(struct record_context *ctx,
 	}
 
 	tip = libinput_event_tablet_tool_get_tip_state(t);
-
-	time = ctx->offset ?
-		libinput_event_tablet_tool_get_time_usec(t) - ctx->offset : 0;
-
+	time = time_offset(ctx, libinput_event_tablet_tool_get_time_usec(t));
 	axes = buffer_tablet_axes(t);
 
 	event->time = time;
@@ -1012,9 +1085,7 @@ buffer_tablet_pad_button_event(struct record_context *ctx,
 		abort();
 	}
 
-	time = ctx->offset ?
-		libinput_event_tablet_pad_get_time_usec(p) - ctx->offset : 0;
-
+	time = time_offset(ctx, libinput_event_tablet_pad_get_time_usec(p));
 	button = libinput_event_tablet_pad_get_button_number(p),
 	state = libinput_event_tablet_pad_get_button_state(p);
 	mode = libinput_event_tablet_pad_get_mode(p);
@@ -1082,9 +1153,7 @@ buffer_tablet_pad_ringstrip_event(struct record_context *ctx,
 		abort();
 	}
 
-	time = ctx->offset ?
-		libinput_event_tablet_pad_get_time_usec(p) - ctx->offset : 0;
-
+	time = time_offset(ctx, libinput_event_tablet_pad_get_time_usec(p));
 	mode = libinput_event_tablet_pad_get_mode(p);
 
 	event->time = time;
@@ -1119,9 +1188,7 @@ buffer_switch_event(struct record_context *ctx,
 		abort();
 	}
 
-	time = ctx->offset ?
-		libinput_event_switch_get_time_usec(s) - ctx->offset : 0;
-
+	time = time_offset(ctx, libinput_event_switch_get_time_usec(s));
 	sw = libinput_event_switch_get_switch(s);
 	state = libinput_event_switch_get_switch_state(s);
 
@@ -1264,7 +1331,7 @@ print_cached_events(struct record_context *ctx,
 
 		switch (e->type) {
 		case EVDEV:
-			print_evdev_event(ctx, &e->u.evdev);
+			print_evdev_event(ctx, d, &e->u.evdev);
 			break;
 		case LIBINPUT:
 			iprintf(ctx, "- %s\n", e->u.libinput.msg);
@@ -2215,6 +2282,8 @@ init_device(struct record_context *ctx, char *path)
 	}
 
 	rc = libevdev_new_from_fd(fd, &d->evdev);
+	if (rc == 0)
+		rc = libevdev_new_from_fd(fd, &d->evdev_prev);
 	if (rc != 0) {
 		fprintf(stderr,
 			"Failed to create context for %s (%s)\n",
